# Intrinio API
#
# Welcome to the Intrinio API! Through our Financial Data Marketplace, we offer a wide selection of financial data feed APIs sourced by our own proprietary processes as well as from many data vendors. For a complete API request / response reference please view the [Intrinio API documentation](https://intrinio.com/documentation/api_v2). If you need additional help in using the API, please visit the [Intrinio website](https://intrinio.com) and click on the chat icon in the lower right corner.
#
# OpenAPI spec version: 2.8.4
# 
# Generated by: https://github.com/swagger-api/swagger-codegen.git

#' @title Crypto operations
#' @description IntrinioSDK.Crypto
#'
#' @field path Stores url path of the request.
#' @field apiClient Handles the client-server communication.
#' @field userAgent Set the user agent of the request.
#'
#' @importFrom R6 R6Class
#'
#' @section Methods:
#' \describe{
#'
#' get_crypto_book_asks Crypto Book Asks
#'
#'
#' get_crypto_book_bids Crypto Book Bids
#'
#'
#' get_crypto_book_summary Crypto Book Summary
#'
#'
#' get_crypto_currencies Crypto Currencies
#'
#'
#' get_crypto_exchanges Crypto Exchanges
#'
#'
#' get_crypto_pairs Crypto Pairs
#'
#'
#' get_crypto_price_technicals_adi Accumulation/Distribution Index
#'
#'
#' get_crypto_price_technicals_adtv Average Daily Trading Volume
#'
#'
#' get_crypto_price_technicals_adx Average Directional Index
#'
#'
#' get_crypto_price_technicals_ao Awesome Oscillator
#'
#'
#' get_crypto_price_technicals_atr Average True Range
#'
#'
#' get_crypto_price_technicals_bb Bollinger Bands
#'
#'
#' get_crypto_price_technicals_cci Commodity Channel Index
#'
#'
#' get_crypto_price_technicals_cmf Chaikin Money Flow
#'
#'
#' get_crypto_price_technicals_dc Donchian Channel
#'
#'
#' get_crypto_price_technicals_dpo Detrended Price Oscillator
#'
#'
#' get_crypto_price_technicals_eom Ease of Movement
#'
#'
#' get_crypto_price_technicals_fi Force Index
#'
#'
#' get_crypto_price_technicals_ichimoku Ichimoku Kinko Hyo
#'
#'
#' get_crypto_price_technicals_kc Keltner Channel
#'
#'
#' get_crypto_price_technicals_kst Know Sure Thing
#'
#'
#' get_crypto_price_technicals_macd Moving Average Convergence Divergence
#'
#'
#' get_crypto_price_technicals_mfi Money Flow Index
#'
#'
#' get_crypto_price_technicals_mi Mass Index
#'
#'
#' get_crypto_price_technicals_nvi Negative Volume Index
#'
#'
#' get_crypto_price_technicals_obv On-balance Volume
#'
#'
#' get_crypto_price_technicals_obv_mean On-balance Volume Mean
#'
#'
#' get_crypto_price_technicals_rsi Relative Strength Index
#'
#'
#' get_crypto_price_technicals_sma Simple Moving Average
#'
#'
#' get_crypto_price_technicals_sr Stochastic Oscillator
#'
#'
#' get_crypto_price_technicals_trix Triple Exponential Average
#'
#'
#' get_crypto_price_technicals_tsi True Strength Index
#'
#'
#' get_crypto_price_technicals_uo Ultimate Oscillator
#'
#'
#' get_crypto_price_technicals_vi Vortex Indicator
#'
#'
#' get_crypto_price_technicals_vpt Volume-price Trend
#'
#'
#' get_crypto_price_technicals_vwap Volume Weighted Average Price
#'
#'
#' get_crypto_price_technicals_wr Williams %R
#'
#'
#' get_crypto_prices Crypto Prices
#'
#'
#' get_crypto_snapshot Crypto Snapshot
#'
#'
#' get_crypto_stats Crypto Stats
#'
#' }
#'
#' @export
CryptoApi <- R6::R6Class(
  'CryptoApi',
  public = list(
    userAgent = "Swagger-Codegen/0.0.1/r",
    apiClient = NULL,
    initialize = function(apiClient){
      if (!missing(apiClient)) {
        self$apiClient <- apiClient
      }
      else {
        self$apiClient <- ApiClient$new()
      }
    },
    get_crypto_book_asks = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      urlPath <- "/crypto/book/asks"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoBookAsks" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBookAsks" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBookAsks" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoBookAsks" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoBookAsks$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_book_bids = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      urlPath <- "/crypto/book/bids"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoBookBids" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBookBids" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBookBids" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoBookBids" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoBookBids$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_book_summary = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('levels' %in% names(opts)) {
        queryParams['levels'] <- opts['levels']
      }
      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      urlPath <- "/crypto/book"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoBook" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBook" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBook" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoBook" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoBook$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_currencies = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      urlPath <- "/crypto/currencies"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoCurrencies" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoCurrencies" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoCurrencies" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoCurrencies" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoCurrencies$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_exchanges = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      urlPath <- "/crypto/exchanges"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoExchanges" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoExchanges" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoExchanges" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoExchanges" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoExchanges$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_pairs = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/pairs"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoPairs" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoPairs" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoPairs" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoPairs" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoPairs$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_adi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/adi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoAccumulationDistributionIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAccumulationDistributionIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAccumulationDistributionIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoAccumulationDistributionIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoAccumulationDistributionIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_adtv = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/adtv"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoAverageDailyTradingVolume" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageDailyTradingVolume" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageDailyTradingVolume" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoAverageDailyTradingVolume" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoAverageDailyTradingVolume$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_adx = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/adx"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoAverageDirectionalIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageDirectionalIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageDirectionalIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoAverageDirectionalIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoAverageDirectionalIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_ao = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('short_period' %in% names(opts)) {
        queryParams['short_period'] <- opts['short_period']
      }
      if ('long_period' %in% names(opts)) {
        queryParams['long_period'] <- opts['long_period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/ao"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoAwesomeOscillator" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAwesomeOscillator" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAwesomeOscillator" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoAwesomeOscillator" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoAwesomeOscillator$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_atr = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/atr"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoAverageTrueRange" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageTrueRange" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoAverageTrueRange" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoAverageTrueRange" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoAverageTrueRange$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_bb = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('standard_deviations' %in% names(opts)) {
        queryParams['standard_deviations'] <- opts['standard_deviations']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/bb"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoBollingerBands" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBollingerBands" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoBollingerBands" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoBollingerBands" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoBollingerBands$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_cci = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('constant' %in% names(opts)) {
        queryParams['constant'] <- opts['constant']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/cci"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoCommodityChannelIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoCommodityChannelIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoCommodityChannelIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoCommodityChannelIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoCommodityChannelIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_cmf = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/cmf"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoChaikinMoneyFlow" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoChaikinMoneyFlow" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoChaikinMoneyFlow" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoChaikinMoneyFlow" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoChaikinMoneyFlow$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_dc = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/dc"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoDonchianChannel" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoDonchianChannel" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoDonchianChannel" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoDonchianChannel" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoDonchianChannel$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_dpo = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/dpo"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoDetrendedPriceOscillator" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoDetrendedPriceOscillator" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoDetrendedPriceOscillator" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoDetrendedPriceOscillator" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoDetrendedPriceOscillator$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_eom = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/eom"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoEaseOfMovement" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoEaseOfMovement" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoEaseOfMovement" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoEaseOfMovement" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoEaseOfMovement$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_fi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/fi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoForceIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoForceIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoForceIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoForceIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoForceIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_ichimoku = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('low_period' %in% names(opts)) {
        queryParams['low_period'] <- opts['low_period']
      }
      if ('medium_period' %in% names(opts)) {
        queryParams['medium_period'] <- opts['medium_period']
      }
      if ('high_period' %in% names(opts)) {
        queryParams['high_period'] <- opts['high_period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/ichimoku"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoIchimokuKinkoHyo" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoIchimokuKinkoHyo" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoIchimokuKinkoHyo" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoIchimokuKinkoHyo" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoIchimokuKinkoHyo$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_kc = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/kc"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoKeltnerChannel" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoKeltnerChannel" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoKeltnerChannel" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoKeltnerChannel" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoKeltnerChannel$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_kst = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('roc1' %in% names(opts)) {
        queryParams['roc1'] <- opts['roc1']
      }
      if ('roc2' %in% names(opts)) {
        queryParams['roc2'] <- opts['roc2']
      }
      if ('roc3' %in% names(opts)) {
        queryParams['roc3'] <- opts['roc3']
      }
      if ('roc4' %in% names(opts)) {
        queryParams['roc4'] <- opts['roc4']
      }
      if ('sma1' %in% names(opts)) {
        queryParams['sma1'] <- opts['sma1']
      }
      if ('sma2' %in% names(opts)) {
        queryParams['sma2'] <- opts['sma2']
      }
      if ('sma3' %in% names(opts)) {
        queryParams['sma3'] <- opts['sma3']
      }
      if ('sma4' %in% names(opts)) {
        queryParams['sma4'] <- opts['sma4']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/kst"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoKnowSureThing" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoKnowSureThing" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoKnowSureThing" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoKnowSureThing" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoKnowSureThing$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_macd = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('fast_period' %in% names(opts)) {
        queryParams['fast_period'] <- opts['fast_period']
      }
      if ('slow_period' %in% names(opts)) {
        queryParams['slow_period'] <- opts['slow_period']
      }
      if ('signal_period' %in% names(opts)) {
        queryParams['signal_period'] <- opts['signal_period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/macd"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoMovingAverageConvergenceDivergence" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMovingAverageConvergenceDivergence" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMovingAverageConvergenceDivergence" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoMovingAverageConvergenceDivergence" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoMovingAverageConvergenceDivergence$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_mfi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/mfi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoMoneyFlowIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMoneyFlowIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMoneyFlowIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoMoneyFlowIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoMoneyFlowIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_mi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('ema_period' %in% names(opts)) {
        queryParams['ema_period'] <- opts['ema_period']
      }
      if ('sum_period' %in% names(opts)) {
        queryParams['sum_period'] <- opts['sum_period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/mi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoMassIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMassIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoMassIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoMassIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoMassIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_nvi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/nvi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoNegativeVolumeIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoNegativeVolumeIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoNegativeVolumeIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoNegativeVolumeIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoNegativeVolumeIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_obv = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/obv"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoOnBalanceVolume" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoOnBalanceVolume" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoOnBalanceVolume" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoOnBalanceVolume" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoOnBalanceVolume$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_obv_mean = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/obv_mean"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoOnBalanceVolumeMean" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoOnBalanceVolumeMean" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoOnBalanceVolumeMean" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoOnBalanceVolumeMean" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoOnBalanceVolumeMean$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_rsi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/rsi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoRelativeStrengthIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoRelativeStrengthIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoRelativeStrengthIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoRelativeStrengthIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoRelativeStrengthIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_sma = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/sma"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoSimpleMovingAverage" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoSimpleMovingAverage" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoSimpleMovingAverage" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoSimpleMovingAverage" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoSimpleMovingAverage$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_sr = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('signal_period' %in% names(opts)) {
        queryParams['signal_period'] <- opts['signal_period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/sr"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoStochasticOscillator" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoStochasticOscillator" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoStochasticOscillator" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoStochasticOscillator" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoStochasticOscillator$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_trix = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/trix"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoTripleExponentialAverage" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoTripleExponentialAverage" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoTripleExponentialAverage" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoTripleExponentialAverage" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoTripleExponentialAverage$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_tsi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('low_period' %in% names(opts)) {
        queryParams['low_period'] <- opts['low_period']
      }
      if ('high_period' %in% names(opts)) {
        queryParams['high_period'] <- opts['high_period']
      }
      if ('price_key' %in% names(opts)) {
        queryParams['price_key'] <- opts['price_key']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/tsi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoTrueStrengthIndex" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoTrueStrengthIndex" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoTrueStrengthIndex" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoTrueStrengthIndex" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoTrueStrengthIndex$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_uo = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('short_period' %in% names(opts)) {
        queryParams['short_period'] <- opts['short_period']
      }
      if ('medium_period' %in% names(opts)) {
        queryParams['medium_period'] <- opts['medium_period']
      }
      if ('long_period' %in% names(opts)) {
        queryParams['long_period'] <- opts['long_period']
      }
      if ('short_weight' %in% names(opts)) {
        queryParams['short_weight'] <- opts['short_weight']
      }
      if ('medium_weight' %in% names(opts)) {
        queryParams['medium_weight'] <- opts['medium_weight']
      }
      if ('long_weight' %in% names(opts)) {
        queryParams['long_weight'] <- opts['long_weight']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/uo"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoUltimateOscillator" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoUltimateOscillator" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoUltimateOscillator" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoUltimateOscillator" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoUltimateOscillator$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_vi = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/vi"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoVortexIndicator" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVortexIndicator" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVortexIndicator" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoVortexIndicator" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoVortexIndicator$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_vpt = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/vpt"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoVolumePriceTrend" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVolumePriceTrend" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVolumePriceTrend" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoVolumePriceTrend" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoVolumePriceTrend$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_vwap = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/vwap"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoVolumeWeightedAveragePrice" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVolumeWeightedAveragePrice" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoVolumeWeightedAveragePrice" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoVolumeWeightedAveragePrice" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoVolumeWeightedAveragePrice$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_price_technicals_wr = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if ('period' %in% names(opts)) {
        queryParams['period'] <- opts['period']
      }
      if ('timeframe' %in% names(opts)) {
        queryParams['timeframe'] <- opts['timeframe']
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices/technicals/wr"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoWilliamsR" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoWilliamsR" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoWilliamsR" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoWilliamsR" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoWilliamsR$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_prices = function(timeframe, opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      if (!missing(`timeframe`)) {
        queryParams['timeframe'] <- `timeframe`
      }
      if ('timezone' %in% names(opts)) {
        queryParams['timezone'] <- opts['timezone']
      }
      if ('start_date' %in% names(opts)) {
        queryParams['start_date'] <- opts['start_date']
      }
      if ('start_time' %in% names(opts)) {
        queryParams['start_time'] <- opts['start_time']
      }
      if ('end_date' %in% names(opts)) {
        queryParams['end_date'] <- opts['end_date']
      }
      if ('end_time' %in% names(opts)) {
        queryParams['end_time'] <- opts['end_time']
      }
      if ('page_size' %in% names(opts)) {
        queryParams['page_size'] <- opts['page_size']
      }
      if ('next_page' %in% names(opts)) {
        queryParams['next_page'] <- opts['next_page']
      }
      urlPath <- "/crypto/prices"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoPrices" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoPrices" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoPrices" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoPrices" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoPrices$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_snapshot = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('pair' %in% names(opts)) {
        queryParams['pair'] <- opts['pair']
      }
      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      urlPath <- "/crypto/snapshot"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoSnapshot" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoSnapshot" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoSnapshot" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoSnapshot" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoSnapshot$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    },
    get_crypto_stats = function(opts = list()){
      queryParams <- list()
      headerParams <- character()

      if ('exchange' %in% names(opts)) {
        queryParams['exchange'] <- opts['exchange']
      }
      if ('currency' %in% names(opts)) {
        queryParams['currency'] <- opts['currency']
      }
      urlPath <- "/crypto/stats"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body)

      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        result <- httr::content(resp, "text", encoding = "UTF-8")

        if ("ApiResponseCryptoStats" == "Numeric") {
          returnObject <- as.numeric(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoStats" == "Integer") {
          returnObject <- as.integer(jsonlite::fromJSON(result))
        } else if ("ApiResponseCryptoStats" == "Character") {
          returnObject <- gsub("\\\"", "", result)
        } else if ("ApiResponseCryptoStats" == "Logical") {
          returnObject <- as.logical(jsonlite::fromJSON(result))
        } else {
          returnObject <- ApiResponseCryptoStats$new()
          returnObject$fromJSONString(result)
        }

        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        result <- jsonlite::fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(result, resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        result <- httr::content(resp, "text", encoding = "UTF-8")
        Response$new(result, resp)
      }

    }
  )
)
